/* -*-c-*- */

/*
  lex input file for source code formatting of Python.

  by Robert Helgesson <rycee@home.se>
*/

%{
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include "markup.h"
#include "syntax.h"
#include "format.h"

#define YY_NO_UNPUT 1

#define PYTHON_EXCEPTION "python_exception"

/* these macros will end up saving us a lot of space. */

#define CAT     sh_cats(&gData,format_python_yytext)
#define XPRE(a) sh_cats(&gData,"<font class=\"%s\">",(a))
#define PRE(a)  XPRE(a); CAT
#define XCAT(a) sh_cats(&gData,(a))
#define POST    XCAT("</font>")
#define WRAP(a) PRE(a); POST

#define CATC      sh_catc(&gData,format_python_yytext[0]); format_python_yytext++;
#define IWRAP(a)  CATC; WRAP(a)

static sh_data gData;

char *python_format ( const char *str )
{
  YY_BUFFER_STATE  yybs   = NULL;
  char            *output = NULL;

  format_python_yy_scan_string(str);
  XCAT(START_TEXT);
  format_python_yylex();
  XCAT(END_TEXT);

  if ( gData.buf ) output = strdup(gData.buf);

  yy_delete_buffer(yybs);
  sh_reset(&gData);
  return output;
}

%}

%option noyywrap
%option prefix="format_python_yy"

WS [ \t]
BOUND [^a-zA-Z0-9_]

KEYWORD and|assert|del|exec|global|in|is|lamdba|not|or|print|yield
KEYWORD_FLOW if|elif|else|while|for|raise|try|except|finally|break|return|continue|pass
TYPE_SPEC class|def
PREPROC_IMPORT from|import
EXCEPTION ArithmeticError|AssertionError|AttributeError|EOFError|EnvironmentError|Exception|FloatingPointError|IOError|ImportError|IndexError|KeyError|KeyboardInterrupt|LookupError|MemoryError|NameError|NotImplementedError|OSError|OverflowError|RuntimeError|StandardError|SyntaxError|SystemError|SystemExit|TypeError|ValueError|ZeroDivisionError

%s quote
%s squote
%s long_quote
%s long_squote

%%

    /********************
     * General
     ********************/

("http://"|"mailto:"|"news://"|"ftp://"|"gopher://")[^<>\"\'%|\\^\{\}` \n\t]* {
  XCAT("<a href=\""); CAT; XCAT("\">"); CAT; XCAT("</a>");
}

    /********************
     * Comments
     ********************/

<INITIAL>#.*          { WRAP(COMMENT); }

    /********************
     * Preprocessor (not really but close 'nuff)
     ********************/

<INITIAL>^{WS}*{PREPROC_IMPORT}[^#\n]*    { WRAP(PREPROC); }

    /********************
     * Constants
     ********************/

<INITIAL>"\""         { PRE(STRING_LITERAL); BEGIN(quote); }
<quote>{
  "\\\""              { CAT; }
  "\\\\"              { CAT; }
  "\""                { CATC; POST; BEGIN(INITIAL); }
}

<INITIAL>"\'"         { PRE(STRING_LITERAL); BEGIN(squote); }
<squote>{
  "\\\'"              { CAT; }
  "\\\\"              { CAT; }
  "\'"                { CATC; POST; BEGIN(INITIAL); }
}

<INITIAL>(\"{3})      { PRE(STRING_LITERAL); BEGIN(long_quote); }
<long_quote>(\"{3})   { CAT; POST; BEGIN(INITIAL); }

<INITIAL>(\'{3})      { PRE(STRING_LITERAL); BEGIN(long_squote); }
<long_squote>(\'{3})  { CAT; POST; BEGIN(INITIAL); }

    /********************
     * Python syntax
     ********************/

<INITIAL>^{WS}*{KEYWORD_FLOW}/{BOUND}       { WRAP(KEYWORD_FLOW);       }
<INITIAL>{BOUND}{KEYWORD}/{BOUND}           { IWRAP(KEYWORD);           }
<INITIAL>^{WS}*{TYPE_SPEC}/{WS}             { WRAP(TYPE_SPECIFIER);     }
<INITIAL>{BOUND}{EXCEPTION}/{BOUND}         { IWRAP(PYTHON_EXCEPTION);  }

    /********************
     * Whatever is left
     ********************/

\n                    { CATC; }
.                     { CATC; }
