/* -*-c-*- */

/*
  lex input file for source code formatting of Verilog.
*/

%{
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include "markup.h"
#include "syntax.h"
#include "format.h"

#define VERILOG_KW "verilog_kw"
#define VERILOG_BT "verilog_bt"

#define YY_NO_UNPUT 1

/* these macros will end up saving us a lot of space. */

#define CAT     sh_cats(&gData,format_verilog_yytext)
#define XPRE(a) sh_cats(&gData,"<FONT class=\"%s\">",(a))
#define PRE(a)  XPRE(a); CAT
#define XCAT(a) sh_cats(&gData,(a))
#define POST    XCAT("</FONT>")
#define WRAP(a) PRE(a); POST

#define CATC sh_catc(&gData,format_verilog_yytext[0]); format_verilog_yytext++;
#define IWRAP(a)  CATC; WRAP(a)

static sh_data gData;

char *verilog_format ( const char *str )
{
  YY_BUFFER_STATE  yybs   = NULL;
  char            *output = NULL;

  format_verilog_yy_scan_string(str);
  XCAT(START_TEXT);
  format_verilog_yylex();
  XCAT(END_TEXT);

  if ( gData.buf ) output = strdup(gData.buf);

  yy_delete_buffer(yybs);
  sh_reset(&gData);
  return output;
}


%}

%option stack
%option noyywrap
%option prefix="format_verilog_yy"

WS [ \t]
BOUND [^a-zA-Z0-9_"']

BASIC_TYPE input|output|parameter|reg|inout|integer|specparam|wire
KEYWORD_FLOW always|and|begin|else|end|endattribute|endcase|endmodule|endfunction|endprimitive|endspecify|endtable|endtask|for|forever|if|initial|macromodule|module|not|or|while
KEYWORD assign|attribute|buf|bufif0|bufif1|case|casex|casez|cmos|deassign|default|defparam|disable|edge|event|force|fork|function|highz0|highz1|join|large|medium|nand|negedge|nmos|nor|notif0|notif1|pmos|posedge|primitive|pull0|pull1|pullup|pulldown|rcmos|release|repeat|rnmos|rpmos|rtran|rtranif0|rtranif1|scalared|small|specify|strength|strong0|strong1|supply0|supply1|table|task|time|tran|tranif0|tranif1|tri|tri0|tri1|triand|trior|trireg|use|vectored|wait|wand|weak0|weak1|wor|xnor|xor
VERILOG_KW $bitstoreal|$countdrivers|$display|$dumpall|$dumpfile|$dumpflush|$dumpoff|$dumpon|$dumpvars|$fclose|$fdisplay|$finish|$fmonitor|$fopen|$fstrobe|$fwrite|$getpattern|$history|$hold|$incsave|$input|$itor|$key|$list|$log|$monitor|$monitoroff|$monitoron|$nokey|$nolog|$period|$printtimescale|$readmemb|$readmemh|$realtime|$realtobits|$recovery|$reset|$reset_count|$reset_value|$restart|$rtoi|$save|$scale|$scope|$setup|$setuphold|$showscopes|$showvariables|$showvars|$skew|$sreadmemb|$sreadmemh|$stime|$stop|$strobe|$time|$timeformat|$width|$write
VERILOG_BT `resetall|`define|`timescale|`ifdef|`else|`endif|`include|`signed|`unsigned|`celldefine|`endcelldefine|`default_nettype|`unconnected_drive|`nounconnected_drive|`accelerate|`noaccelerate|`protect|`endprotect|`protected|`endprotected|`expand_vectornets|`noexpand_vectornets|`autoexpand_vectornets|`remove_gatenames|`noremove_gatenames|`remove_netnames|`noremove_netnames


%s ccomment
%s quote

%%

    /********************
     * General
     ********************/

("http://"|"mailto:"|"news://"|"ftp://"|"gopher://")[^<>\"\'%|\\^\{\}` \n\t]* {
  XCAT("<a href=\""); CAT; XCAT("\">"); CAT; XCAT("</a>");
}

    /********************
     * Comments
     ********************/

<INITIAL>"//".*   { WRAP(COMMENT); }

<INITIAL>"/*"   { PRE(COMMENT); yy_push_state(ccomment); }
<ccomment>"*/"  { CAT; POST; yy_pop_state(); }

    /********************
     * Constants
     ********************/

<INITIAL>"\""   { PRE(STRING_LITERAL); yy_push_state(quote); }
<quote>{
  "\\\""        { CAT; }
  "\\\\"        { CAT; }
  "\""          { CATC; POST; yy_pop_state(); }
}

    /********************
     * Verilog syntax
     ********************/

<INITIAL>{BOUND}{BASIC_TYPE}/{BOUND}      { IWRAP(BASIC_TYPE);        }
<INITIAL>^{WS}*{KEYWORD_FLOW}/{BOUND}     { WRAP(KEYWORD_FLOW);       }
<INITIAL>{BOUND}{KEYWORD_FLOW}/{BOUND}    { IWRAP(KEYWORD_FLOW);      }
<INITIAL>{BOUND}{KEYWORD}/{BOUND}         { IWRAP(KEYWORD);           }
<INITIAL>{BOUND}{VERILOG_KW}/{BOUND}      { IWRAP(VERILOG_KW);        }
<INITIAL>^{WS}*{VERILOG_BT}.*             { WRAP(VERILOG_BT);         }

    /********************
     * Whatever is left
     ********************/

\n             { CATC; }
.              { CATC; }
