/* -*-c-*- */

/*
  lex input file for source code formatting of VHDL.
*/

%{
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include "markup.h"
#include "syntax.h"
#include "format.h"

#define VHDL_FUNCTION   "vhdl_function"
#define VHDL_PACKAGE    "vhdl_package"
#define VHDL_ATTRIBUTE  "vhdl_attribute"
#define VHDL_ENUM       "vhdl_enum"
#define VHDL_CONSTANT   "vhdl_constant"

#define YY_NO_UNPUT 1

/* these macros will end up saving us a lot of space. */

#define CAT     sh_cats(&gData,format_vhdl_yytext)
#define XPRE(a) sh_cats(&gData,"<FONT class=\"%s\">",(a))
#define PRE(a)  XPRE(a); CAT
#define XCAT(a) sh_cats(&gData,(a))
#define POST    XCAT("</FONT>")
#define WRAP(a) PRE(a); POST

#define CATC      sh_catc(&gData,format_vhdl_yytext[0]); format_vhdl_yytext++;
#define IWRAP(a)  CATC; WRAP(a)

static sh_data gData;

char *vhdl_format ( const char *str )
{
  YY_BUFFER_STATE  yybs   = NULL;
  char            *output = NULL;

  format_vhdl_yy_scan_string(str);
  XCAT(START_TEXT);
  format_vhdl_yylex();
  XCAT(END_TEXT);

  if ( gData.buf ) output = strdup(gData.buf);

  yy_delete_buffer(yybs);
  sh_reset(&gData);
  return output;
}


%}

%option stack
%option noyywrap
%option prefix="format_vhdl_yy"

WS [ \t]
BOUND [^a-zA-Z0-9_"']

VHDL_FUNCTION now|resolved|rising_edge|falling_edge|read|readline|write|writeline|endfile|resize|is_X|std_match|shift_left|shift_right|rotate_left|rotate_right|to_unsigned|to_signed|to_integer|to_stdLogicVector|to_stdULogic|to_stdULogicVector|to_bit|to_bitVector|to_X01|to_X01Z|to_UX01|to_01|conv_unsigned|conv_signed|conv_integer|conv_std_logic_vector|shl|shr|ext|sxt|frequency|sign|ceil|floor|round|trunc|fmax|fmin|uniform|sqrt|cbrt|exp|log|sin|cos|tan|arcsin|arccos|arctan|sinh|cosh|tanh|arcsinh|arccosh|arctanh|cmplx|complex_to_polar|polar_to_complex|arg|conj
BASIC_TYPE boolean|bit|bit_vector|character|severity_level|integer|real|time|natural|positive|string'|line|text|side|unsigned|signed|delay_length|file_open_kind|file_open_status|std_logic|std_logic_vector|std_ulogic|std_ulogic_vector|domain_type|real_vector|complex|complex_polar
KEYWORD_FLOW after|and|begin|else|elsif|end|exit|for|if|next|not|or|return|then|to|until|when|while
KEYWORD abs|access|alias|all|architecture|array|assert|attribute|block|body|buffer|bus|case|component|configuration|constant|disconnect|downto|entity|file|function|generate|generic|group|guarded|impure|in|inertial|inout|is|label|library|linkage|literal|loop|map|mod|nand|new|next|nor|null|of|on|open|others|out|package|port|postponed|procedure|process|pure|range|record|register|reject|rem|report|rol|ror|select|severity|shared|signal|sla|sll|sra|srl|subtype|transport|type|unaffected|units|use|variable|wait|with|xnor|xor|across|break|limit|nature|noise|procedural|quantity|reference|spectrum|subnature|terminal|through|tolerance|Use
VHDL_PACKAGE std_logic_1164|numeric_std|numeric_bit|standard|textio|std_logic_arith|std_logic_signed|std_logic_unsigned|std_logic_misc|std_logic_textio|ieee|std|work|math_real|math_complex
VHDL_ATTRIBUTE base|left|right|high|low|pos|val|succ|pred|leftof|rightof|range|reverse_range|length|delayed|stable|quiet|transaction|event|active|last_event|last_active|last_value|driving|driving_value|ascending|value|image|simple_name|instance_name|path_name|foreign|across|through|reference|contribution|tolerance|dot|integ|delayed|above|zoh|ltf|ztf|ramp|slew
VHDL_ENUM true|false|note|warning|error|failure|read_mode|write_mode|append_mode|open_ok|status_error|name_error|mode_error|fs|ps|ns|us|ms|sec|min|hr|right|left|quiescent_domain|time_domain|frequency_domain
VHDL_CONSTANT math_e|math_1_over_e|math_pi|math_two_pi|math_1_over_pi|math_half_pi|math_q_pi|math_3_half_pi|math_log_of_2|math_log_of_10|math_log2_of_e|math_log10_of_e|math_sqrt2|math_sqrt1_2|math_sqrt_pi|math_deg_to_rad|math_rad_to_deg|cbase_1|cbase_j|czero

%s quote

%%

    /********************
     * General
     ********************/

("http://"|"mailto:"|"news://"|"ftp://"|"gopher://")[^<>\"\'%|\\^\{\}` \n\t]* {
  XCAT("<a href=\""); CAT; XCAT("\">"); CAT; XCAT("</a>");
}

    /********************
     * Comments
     ********************/

<INITIAL>"--".*   { WRAP(COMMENT); }

    /********************
     * Constants
     ********************/

<INITIAL>"\""   { PRE(STRING_LITERAL); yy_push_state(quote); }
<quote>{
  "\\\""        { CAT; }
  "\\\\"        { CAT; }
  "\""          { CATC; POST; yy_pop_state(); }
}

    /********************
     * VHDL syntax
     ********************/

<INITIAL>{BOUND}{VHDL_FUNCTION}/{BOUND}       { IWRAP(VHDL_FUNCTION);    }
<INITIAL>{BOUND}{BASIC_TYPE}/{BOUND}          { IWRAP(BASIC_TYPE);       }
<INITIAL>{BOUND}{KEYWORD}/{BOUND}             { IWRAP(KEYWORD);          }
<INITIAL>^{WS}*{KEYWORD_FLOW}/{BOUND}         { WRAP(KEYWORD_FLOW);      }
<INITIAL>{BOUND}{KEYWORD_FLOW}/{BOUND}        { IWRAP(KEYWORD_FLOW);     }
<INITIAL>^{WS}*{VHDL_PACKAGE}/[\.]            { WRAP(VHDL_PACKAGE);      }
<INITIAL>{VHDL_PACKAGE}/[\.]                  { WRAP(VHDL_PACKAGE);      }
<INITIAL>^{WS}*{VHDL_ATTRIBUTE}/[\.]          { WRAP(VHDL_ATTRIBUTE);    }
<INITIAL>{VHDL_ATTRIBUTE}/[\.]                { WRAP(VHDL_ATTRIBUTE);    }
<INITIAL>^{WS}*{VHDL_ENUM}/[\.]               { WRAP(VHDL_ENUM);         }
<INITIAL>{VHDL_ENUM}/[\.]                     { WRAP(VHDL_ENUM);         }
<INITIAL>^{WS}*{VHDL_CONSTANT}/[\.]           { WRAP(VHDL_CONSTANT);     }
<INITIAL>{VHDL_CONSTANT}/[\.]                 { WRAP(VHDL_CONSTANT);     }
'[0-9]+'                                      { WRAP(VHDL_CONSTANT);     }

    /********************
     * Whatever is left
     ********************/

\n             { CATC; }
.              { CATC; }
